/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package ch.educeth.eccapplet;

import java.awt.*;
import java.util.*;


/**
 * Diese Klasse enthaelt das Modell der Farben fuer die verschiedenen Bilder.
 * Falls sich die Farben aendern werden die entsprechenden Observer
 * benachrichtigt.
 */
public class EccColorModel extends Observable {
	private int numberOfDim;
	private Color originalColor[];
	private Color hyperColor[];
	private Color hyperErrorColor[];
	private boolean generatorFlag = true;
	private boolean selfModified = false;
	private String modified1 = "";
	private String modified2 = "";

	private int[][] MATRIX = {
		{0,0,0,0,0,0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0,0,0,0,0,0}
	};

	/* Matrix zur Abbildung eines 13-dimensionalen Vektors in den RGB-Raum */
	private final static int[][] COLORS13 = {
		{  0,  0, 51},
		{  0, 51,  0},
		{ 51,  0,  0},
		{  0, 36, 37},
		{ 35,  0, 37},
		{ 35, 36,  0},
		{ 29, 29, 27},
		{  5,  4, 37},
		{  5, 37,  4},
		{ 39,  4,  5},
		{ 10,  9, 28},
		{ 10, 30,  9},
		{ 29,  9,  8}
	};
	
	/* Ordnet die vier Grundwerte vier 7-dimensionalen Vektoren zu */
	private final static int[][] MATRIX13 = {
		{0,0,0,0,0,0,0,0,0,0,0,0,0},
		{0,1,0,1,1,0,1,1,0,1,0,1,1},
		{1,0,1,1,0,1,0,1,1,0,1,1,0},
		{1,1,1,0,1,1,1,0,1,1,1,0,1}
	};
		
	final static String GENERATOR131 = "1011010110110";
	final static String GENERATOR132 = "0101101101011";
		

	/* Matrix zur Abbildung eines 7-dimensionalen Vektors in den RGB-Raum */
	private final static int[][] COLORS7 = {
   	{  0,  0, 85},
   	{  0, 85,  0},
   	{ 85,  0,  0},
   	{  0, 60, 60},
   	{ 60,  0, 60},
   	{ 60, 60,  0},
   	{ 50, 50, 50}
	};

	/* Ordnet die vier Grundwerte vier 7-dimensionalen Vektoren zu */
	private final static int[][] MATRIX7 = {
		{0,0,0,0,0,0,0},
		{0,1,0,1,1,0,1},
		{1,0,1,1,0,1,0},
		{1,1,1,0,1,1,1}
	};
	
	final static String GENERATOR71 = "1011010";
	final static String GENERATOR72 = "0101101";
	
	/* Matrix zur Abbildung eines 6-dimensionalen Vektors in den RGB-Raum */
	private 	final static int[][] COLORS6 = {
		{  0,  0,105},
		{  0,105,  0},
		{105,  0,  0},
		{  0, 75, 75},
		{ 75,  0, 75},
		{ 75, 75,  0}
	};

	/* Ordnet die vier Grundwerte vier 6-dimensionalen Vektoren zu */
	private final static int[][] MATRIX6 = {
		{0,0,0,0,0,0},
		{0,0,0,1,1,1},
		{1,1,1,0,0,0},
		{1,1,1,1,1,1}
	};

	final static String GENERATOR61 = "111000";
	final static String GENERATOR62 = "000111";

	/* Matrix zur Abbildung eines 5-dimensionalen Vektors in den RGB-Raum */
	private 	final static int[][] COLORS5 = {
		{  0,  0,130},
		{  0,130,  0},
		{130,  0,  0},
		{ 62,  0, 62},
		{ 62, 62,  0}
	};

	/* Ordnet die vier Grundwerte vier 5-dimensionalen Vektoren zu */
	private final static int[][] MATRIX5 = {
		{0,0,0,0,0},
		{0,1,0,1,1},
		{1,0,1,1,0},
		{1,1,1,0,1}
	};

	final static String GENERATOR51 = "10110";
	final static String GENERATOR52 = "01011";

	/* Matrix zur Abbildung eines 4-dimensionalen Vektors in den RGB-Raum */
	private final static int[][] COLORS4 = {
		{  0,  0,160},
		{  0,160,  0},
		{160,  0,  0},
		{ 95, 95, 95}
	};

	/* Ordnet die vier Grundwerte vier 4-dimensionalen Vektoren zu */
	private final static int[][] MATRIX4 = {
		{0,0,0,0},
		{0,1,1,1},
		{1,0,1,0},
		{1,1,0,1}
	};

	final static String GENERATOR41 = "1010";
	final static String GENERATOR42 = "0111";

	/* Matrix zur Abbildung eines 3-dimensionalen Vektors in den RGB-Raum */
	private final static int[][] COLORS3 = {
		{  0,  0,255},
		{  0,255,  0},
		{255,  0,  0}
	};
	
	/* Ordnet die vier Grundwerte(00,01,10,11) vier 3-dimensionalen 
	 * Vektoren zu */
	private final static int[][] MATRIX3 = {
		{0,0,0},
		{1,1,0},
		{1,0,1},
		{0,1,1}
	};

	final static String GENERATOR31 = "101";
	final static String GENERATOR32 = "110";

	/* Enthaelt die einzelnen Color-Matrizen */
	private final static int [][][] COLORS = {
		null, null, null,
		COLORS3,
		COLORS4,
		COLORS5,
		COLORS6,
		COLORS7,
		null, null, null, null, null,
		COLORS13
	};

	/* Enthaelt die vordefinierten Matrizen */
	private final static int [][][] PREMATRIX = {
		null, null, null,
		MATRIX3,
		MATRIX4,
		MATRIX5,
		MATRIX6,
		MATRIX7,
		null, null, null, null, null,
		MATRIX13
	};

	/** Konstruktor, inizialisiert das Farbenmodell */
	public EccColorModel() {
		numberOfDim = 7;
		originalColor = new Color[4];
		originalColor[0] = EccConstants.ORIGINAL_COLOR1;
		originalColor[1] = EccConstants.ORIGINAL_COLOR2;
		originalColor[2] = EccConstants.ORIGINAL_COLOR3;
		originalColor[3] = EccConstants.ORIGINAL_COLOR4;
		hyperColor = new Color[4];
		hyperErrorColor = new Color[8192];
		this.setNumOfDim(7);
	}

	/**
	 * Liefert die Binaerdarstellung einer Zahl
	 * @param number Zahl deren Binaerdarstellung gesucht ist
	 * @param binary Array in welchem die Binaerdarstellung zurueck gegeben wird
	 * @param dimension Enthaelt die Groesse des Arrays 'binary'
	 */
	public static void getBinary(int number, int[] binary, int dimension) {
		for (int i = (dimension - 1); i >= 0; i--) {
			binary[i] = number % 2;
			number = number / 2;
		}
	}	/* getBinary */

	/**
	 * Liefert die Zahl zu einer Binaerdarstellung
	 * @param binary Binaerdarstellung der gesuchten Zahl
	 * @param dimension Enthaelt die Groesse des Arrays 'binary'
	 * @return Liefert die Zahl der Binaerdarstellung
	 */
	public static int getNumber(int[] binary, int dimension) {
		int number = 0, power, up = 0;	
		for (int i = (dimension - 1); i >= 0; i--) {
			power = (int)(Math.pow(2, up));
			up++;
			number = number + binary[i]*power;
		}
		return number;
	}	/* getNumber */

	/**
	 * Liefert die Farbe zu einer Binaerdarstellung, dem Hyperraum entsprechend
	 * @param binaryCombination Binaerdarstellung 
	 * @param dimension Dimension des Hyperraums
	 * @return Farbe zur Binaerdarstellung
	 */
	public static Color getColor (int[] binaryCombination, int dimension) {
		int r = 0, g = 0, b = 0;
		for (int i = 0; i < dimension; i++){
			r += COLORS[dimension][i][0] * binaryCombination[i];
			g += COLORS[dimension][i][1] * binaryCombination[i];
			b += COLORS[dimension][i][2] * binaryCombination[i];
		}
		return new Color (r, g, b);
	}	/* getColor */
	
	/**
	 * Liefert eine der vier Originalfarben
	 * @param value Wert eines Punktes des Bildes
	 * @return Originalfarbe
	 */
	public Color getOriginalColor(int value) {
		return originalColor[value - 1];
	}	/* getOriginalColor */

	/**
	 * Liefert eine von vier Farben im Hyperraum, die den Originalfarben 
	 * entsprechend zugeordnet sind
	 * @param value Wert eines Punktes des Bildes
	 * @return Farbe aus dem Hyperraum
	 */
	public Color getHyperOriginalColor(int value) {
		return hyperColor[value - 1];
	}	/* getHyperOriginalColor */

	/**
	 * Setzt die vier Farben im Hyperraum, entsprechend der Dimension
	 * und der Matrix, welche die Zuordnung regelt
	 * @param dim Dimension des Hyperraums
	 */
	public void setHyperOriginalColor(int dim) {
		for(int i=0; i < 4; i++){
			int[] bin = new int[dim];
			for(int j=0; j < dim; j++){
				bin[j] = MATRIX[i][j];
				}
			hyperColor[i] = getColor(bin, dim);
		}
	}	/* setHyperOriginalColor */

	/**
	 * Setzt eine von vier Farben im Hyperraum, entprechend der Dimension - 
	 * wird verwendet wenn im 'visuellen' Teil die Farben selbst geaendert
	 * werden
	 * @param at Position der Farbe welche geaendert werden soll
	 * @param color neue Farbe
	 */
	public void setHyperOriginalColorAt(int at, int color) {
		int[] bin = new int[numberOfDim];
		getBinary(color, bin, numberOfDim);
		for(int j=0; j < numberOfDim; j++){
			MATRIX[at][j] = bin[j];
		}
		hyperColor[at] = getColor(bin, numberOfDim);
		setChanged();
		notifyObservers();
	}	/* setHyperOriginalColor */

	/**
	 * Liefert eine Farbe aus dem ganzen Hyperraum
	 * @param value Zahl aus dem fehlerhaft uebertragenen Bild
	 * @return Farbe aus dem Hyperraum, die je nach Anzahl Fehler nicht
	 * in den vier ausgewaehlten Farben des Hyperraums vorhanden ist
	 */
	public Color getHyperErrorColor(int value) {
		return hyperErrorColor[value];
	}	/* getHyperErrorColor */

	/**
	 * Liefert eine der vier ausgewaehlten Farben im Hyperraum, welche am 
	 * naechsten liegt
	 * @param binary Binaerdarstellung einer Farbe aus dem Hyperraum
	 * @param dim Dimension des Hyperraums
	 * @return Naechste Farbe zur Binaerdarstellung
	 */
	public Color getNearestColor(int[] binary, int dim) {
		int[] a = new int[dim];
		int[] b = new int[dim];
		int[] c = new int[dim];
		int[] d = new int[dim];
		for (int i = 0; i < dim; i++) {
			a[i] = MATRIX[0][i];
			b[i] = MATRIX[1][i];
			c[i] = MATRIX[2][i];
			d[i] = MATRIX[3][i];
		}
		int x = 0;
		int suma = 0, sumb = 0, sumc = 0, sumd = 0;
		for (int j = 0; j < dim; j++) {
			if (a[j] == binary[j]) {
				suma++;
			} 
			if (b[j] == binary[j]) {
				sumb++;
			} 
			if (c[j] == binary[j]) {
				sumc++;
			} 
			if (d[j] == binary[j]) {
				sumd++;
			}
		}
		if ((suma>=sumb) && (suma>=sumc) && (suma>=sumd)) {
			return originalColor[0];
		}
		if ((sumb>=suma) && (sumb>=sumc) && (sumb>=sumd)) {
			return originalColor[1];
		}
		if ((sumc>=suma) && (sumc>=sumb) && (sumc>=sumd)) {
			return originalColor[2];
		}
		if ((sumd>=suma) && (sumd>=sumb) && (sumd>=sumc)) {
			return originalColor[3];
		}
		return Color.yellow;
	}	/* getNearestColor */
		
	/**
	 * Liefert die Anzahl der Dimensionen des Hyperraums
	 * @return Anzahl der Dimensionen
	 */
	public int getNumOfDim() {
		return numberOfDim;
	}	/* getNumOfDim */
	
	/**
	 * Setzt die Anzahl Dimensionen im Hyperraum und updated dabei die Farbarrays
	 * @param value Anzahl Dimensionen
	 */
	public void setNumOfDim(int value) {
		numberOfDim = value;
		this.setModifiedFlag(false);
		this.setGeneratorFlag(true);
		setPredefinedMatrix(value);
		setHyperOriginalColor(value);
		setHyperErrorPicture(value);
		setChanged();
		notifyObservers();
	}

	/**
	 * Setzt die Matrix, welche die Zuordnung zwichen der Ausgangs- und der
	 * hoeheren Dimension regelt
	 * @param dim Anzahl Dimensionen
	 */
	public void setPredefinedMatrix(int dim) {
		for (int i=0; i < 4; i++) {
			for (int j=0; j < dim; j++) {
				MATRIX[i][j] = PREMATRIX[dim][i][j];
			}
		}
	}	/* setPredefinedMatrix */

	/**
	 * Setzt die Farbtabelle des Hyperraums
	 * @param dim Dimension des Hyperraums
	 */
	public void setHyperErrorPicture(int dim) {
		int colors = (int)Math.pow(2, dim);
		for (int i = 0; i < colors; i++){
			int[] bin = new int[dim];
			getBinary(i, bin, dim);
			hyperErrorColor[i] = getColor(bin, dim);
		}
	}	/* setHyperErrorPicture */

	/**
	 * Liefert den Wert der Matrix an einer bestimmten Stelle
	 * @param x Position in x-Richtung
	 * @param y Position in y-Richtung
	 * @return Wert der Matrix an der Stelle (x, y)
	 */
	public int getMatrixAt(int x, int y) {
		return MATRIX[x][y];
	}	/* getMatrixAt */

	/**
	 * Setzt den Wert der Matrix an einer bestimmten Stelle
	 * @param x Position in x-Richtung
	 * @param y Position in y-Richtung
	 * @param value zu setzender Wert
	 */
	public void setMatrixAt(int x, int y, int value) {
		MATRIX[x][y] = value;
			setChanged();
			notifyObservers();		
	}

	/**
	 * Liefert, ob eine vorgegebene Matrix verwendet wurde oder nicht
	 * @return Wahr, falls eine Originalmatrix verwendet wurde
	 */
	public boolean getGeneratorFlag() {
		return generatorFlag;
	}	/* getGeneratorFlag */

	/**
	 * Setzt das Flag, ob eine vorgegebene Matrix verwendet wurde oder nicht
	 * @param flag Setzte wahr, falls eine Originalmatrix verwendet wurde
	 */
	public void setGeneratorFlag(boolean flag) {
		generatorFlag = flag;
	}	/* setGeneratorFlag */

	/**
	 * Liefert, ob die Generatormatrix veraendert wurde oder nicht
	 * @return Wahr, falls die Generatormatrix veraendert wurde
	 */
	public boolean getModifiedFlag() {
		return selfModified;
	}	/* getModifiedFlag */

	/**
	 * Setzt das Flag, ob die Generatormatrix veraendert wurde oder nicht
 	 * @return flag Setzte wahr, falls die Generatormatrix veraendert wurde
 	 */
	public void setModifiedFlag(boolean flag) {
		selfModified = flag;
	}	/* setModifiedFlag */

	/**
	 * Setzt die Generatormatrix als String
	 * @param str1 erste Zeile der Generatormatrix
	 * @param str2 zweite Zeile der Generatormatrix
	 */
	public void setModifiedString(String str1, String str2) {
		modified1 = str1;
		modified2 = str2;
	}	/* setModifiedString */

	/** Gibt die erste Zeile der Generatormatrix zurueck
	 * @return erste Zeile der Generatormatrix
	 */
	public String getFirstString() {
		return modified1;
	}	/* getFirstString */
	
	/** Gibt die zweite Zeile der Generatormatrix zurueck
	 * @return zweite Zeile der Generatormatrix
	 */	
	public String getSecondString() {
		return modified2;
	}	/* getSecondString */
}
		
