/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package ch.educeth.eccapplet;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;

import ch.educeth.util.gui.dndpanel.LightweightDragDrop;


/**
 * Panel welches die Eigenschaften der Fehler korrigierenden Codes graphisch
 * darstellt.
 */
public class EccColorViewRight extends JPanel implements LightweightDragDrop {
	private Font SMALL_FONT=new Font("SansSerif",Font.BOLD,12);
	private int dragLeft = 0, dragRight = 0, dragUp = 0, dragDown = 0;
	private int p1DropLeft = 0, p1DropRight = 0, p1DropUp = 0, p1DropDown = 0;
	private int p2DropLeft = 0, p2DropRight = 0, p2DropUp = 0, p2DropDown = 0;
	private int p3DropLeft = 0, p3DropRight = 0, p3DropUp = 0, p3DropDown = 0;
	private int p4DropLeft = 0, p4DropRight = 0, p4DropUp = 0, p4DropDown = 0;
	private int iPanelHeight = 0, iPanelWidth = 0;
	private int iDim = 3;
	private int iXGap = 0, iYGap = 0;
	private int iColorIndex = 0;
	private int iSquareSize = 0;
	private int iNrOfSqInX = 1;
	private int iNrOfSqInY = 1;
			
	/**
	 * Zeichnet den Inhalt der 'visuellen' Komponente
	 */
	public void paint(Graphics g){
		super.paint(g);
		g.setFont(SMALL_FONT);
		iPanelHeight = this.getSize().height;
		iPanelWidth = this.getSize().width;
		iDim = EccController.CONTROLLER.ecmo.getNumOfDim();
		/* Kalkulieren der Quadratgroesse */
		int temp1, temp2, temp3 = 0;
		if (iDim == 7) {
			iNrOfSqInY = temp3 = 16;
		} else if ((iDim == 6) || (iDim == 5)) {
			iNrOfSqInY = temp3 = 8;
		} else if ((iDim == 4) || (iDim == 3)) {
			iNrOfSqInY = temp3 = 4;
		} else {
		}
		temp1 = (int)((float)(iPanelHeight/2 - 10)/(float)temp3);
		if ((iDim == 7) || (iDim == 6)) {
			iNrOfSqInX = temp3 = 8;
		} else if ((iDim == 5) || (iDim == 4) || (iDim == 3)) {
			iNrOfSqInX = temp3 = 4;
			if (iDim == 3) {
				iNrOfSqInX = 2;
			}
		} else {
		}
		temp2 = (int)((float)(iPanelWidth - 20)/(float)temp3);
		if (temp1 < temp2) {
			iSquareSize = temp1;
		} else {
			iSquareSize = temp2;
		}
		if (iSquareSize > iPanelWidth/5.2) {
			iSquareSize = (int)(iPanelWidth/5.2 - 10);
		}
		if (iDim == 13) {
			iSquareSize = 3;
			iNrOfSqInX = 64;
			iNrOfSqInY = 128;
		}
		g.setColor(Color.black);
		/* Setzen der erlaubten Bereichs fuer Drag */
		if (iDim != 13) {
			dragLeft = (iPanelWidth - (iSquareSize*iNrOfSqInX))/2;
			dragUp = iPanelHeight/2 + ((iPanelHeight/2 - (iSquareSize*iNrOfSqInY))/2);
			dragRight = dragLeft + (iSquareSize*iNrOfSqInX);
			dragDown = dragUp + (iSquareSize*iNrOfSqInY);
		} else {
			dragLeft = (iPanelWidth - (iSquareSize*iNrOfSqInX))/2;
			dragUp = 135;
			dragRight = dragLeft + (iSquareSize*iNrOfSqInX);
			dragDown = 135 + (iSquareSize*iNrOfSqInY);		
		}
		/* Berechnen der Abstaende zwischen den einzelnen Komponenten */
		iXGap = (int)(iPanelWidth/5.5);
		iYGap = (int)(iPanelHeight/(2*9.5));
		/* Zeile: Original Farben */
		if (iDim != 13) {
			g.drawString(EccConstants.ORIGINAL_COLORS, iXGap, 2*iYGap - 12);
			for(int i = 1; i < 5; i++) {
				Color color = new Color(0,0,0);
				color = EccController.CONTROLLER.ecmo.getOriginalColor(i);
				g.setColor(color);
				g.fillRect(i*iXGap, 2*iYGap, iSquareSize, iSquareSize);
				g.setColor(Color.black);
				g.drawRect(i*iXGap - 1, 2*iYGap - 1, iSquareSize + 1, iSquareSize + 1);
			}
		} else {
			g.drawString(EccConstants.ORIGINAL_COLORS, iXGap, 20);
			for(int i = 1; i < 5; i++) {
				Color color = new Color(0,0,0);
				color = EccController.CONTROLLER.ecmo.getOriginalColor(i);
				g.setColor(color);
				g.fillRect(i*iXGap, 35, iSquareSize, iSquareSize);
				g.setColor(Color.black);
				g.drawRect(i*iXGap - 1, 35 - 1, iSquareSize + 1, iSquareSize + 1);
			}
		}
		/* Zeile: Farben im Hyperraum */
		if (iDim != 13) {
			g.drawString(EccConstants.HYPERSPACE_COLORS, iXGap, 6*iYGap - 12);
			for (int i = 1; i < 5; i++){
				Color color = new Color(0,0,0);
				color = EccController.CONTROLLER.ecmo.getHyperOriginalColor(i);
				g.setColor(color);
				g.fillRect(i*iXGap, 6*iYGap, iSquareSize, iSquareSize);
				g.setColor(Color.black);
				g.drawRect(i*iXGap - 1, 6*iYGap - 1, iSquareSize + 1, iSquareSize + 1);	
			}
		} else {
			g.drawString(EccConstants.HYPERSPACE_COLORS, iXGap, 70);
			for (int i = 1; i < 5; i++) {
				Color color = new Color(0,0,0);
				color = EccController.CONTROLLER.ecmo.getHyperOriginalColor(i);
				g.setColor(color);
				g.fillRect(i*iXGap, 85, iSquareSize, iSquareSize);
				g.setColor(Color.black);
				g.drawRect(i*iXGap - 1, 85 - 1, iSquareSize + 1, iSquareSize + 1);
			}
		}
		/* Setzen des erlaubten Bereichs fuer Drop */
		p1DropLeft = iXGap;
	 	p1DropRight = iXGap + iSquareSize; 
		p1DropUp = 	6*iYGap;
		p1DropDown = 6*iYGap + iSquareSize;
		p2DropLeft = 2*iXGap;
	 	p2DropRight = 2*iXGap + iSquareSize;
		p2DropUp = 	6*iYGap;
		p2DropDown = 6*iYGap + iSquareSize;
		p3DropLeft = 3*iXGap;
	 	p3DropRight = 3*iXGap + iSquareSize; 
		p3DropUp = 	6*iYGap;
		p3DropDown = 6*iYGap + iSquareSize;
		p4DropLeft = 4*iXGap;
	 	p4DropRight = 4*iXGap + iSquareSize; 
		p4DropUp = 	6*iYGap;
		p4DropDown = 6*iYGap + iSquareSize;
		/* for easier drop, because of the small squaresize */
		if (iDim == 13) {
			p1DropLeft = iXGap - 3;
			p1DropRight = iXGap + iSquareSize + 3; 
			p1DropUp = 	85 - 3;
			p1DropDown = 85 + iSquareSize + 3;
			p2DropLeft = 2*iXGap - 3;
		 	p2DropRight = 2*iXGap + iSquareSize + 3; 
			p2DropUp = 	85 - 3;
			p2DropDown = 85 + iSquareSize + 3;
			p3DropLeft = 3*iXGap - 3;
	 		p3DropRight = 3*iXGap + iSquareSize + 3; 
			p3DropUp = 	85 - 3;
			p3DropDown = 85 + iSquareSize + 3;
			p4DropLeft = 4*iXGap - 3;
		 	p4DropRight = 4*iXGap + iSquareSize + 3; 
			p4DropUp = 	85 - 3;
			p4DropDown = 85 + iSquareSize + 3;
		}
		if (iDim != 13) {
			g.setColor(Color.black);
			/* Zeile: Ganzer Hyperraum */
			g.drawString(EccConstants.WHOLE_HYPERSPACE, iXGap, dragUp - 12);
			int count = 0;
			for (int i=0; i < iNrOfSqInY; i++) {
				for (int j=0; j < iNrOfSqInX; j++) {
					Color color = new Color(0,0,0);
					color = EccController.CONTROLLER.ecmo.getHyperErrorColor(count);
					count++;
					g.setColor(color);
					g.fillRect(dragLeft + j*iSquareSize, dragUp + i*iSquareSize,
								  iSquareSize, iSquareSize);
					if (((color.equals(EccController.CONTROLLER.ecmo.getHyperOriginalColor(1))) ||
						  (color.equals(EccController.CONTROLLER.ecmo.getHyperOriginalColor(2))) ||
						  (color.equals(EccController.CONTROLLER.ecmo.getHyperOriginalColor(3))) ||
						  (color.equals(EccController.CONTROLLER.ecmo.getHyperOriginalColor(4)))) &&
						  (iDim > 3)) {
						 g.setColor(Color.black);
						 g.drawRect(dragLeft + j*iSquareSize, dragUp + i*iSquareSize, 
						 				iSquareSize - 1, iSquareSize - 1);
					}
				}
			}
			g.setColor(Color.black);
			g.drawRect((iPanelWidth - (iSquareSize*iNrOfSqInX))/2 - 1,
				  iPanelHeight/2 + ((iPanelHeight/2 - (iSquareSize*iNrOfSqInY))/2 - 1),
				  iSquareSize*iNrOfSqInX + 1, iSquareSize*iNrOfSqInY + 1);
		} else {
			g.setColor(Color.black);
			/* Zeile: Ganzer Hyperraum */
			g.drawString(EccConstants.WHOLE_HYPERSPACE, iXGap, 120);
			int count = 0;
			for (int i=0; i < iNrOfSqInY; i++) {
				for (int j=0; j < iNrOfSqInX; j++) {
					Color color = new Color(0,0,0);
					color = EccController.CONTROLLER.ecmo.getHyperErrorColor(count);
					int[] binary = new int[13];  
					EccController.CONTROLLER.ecmo.getBinary(count, binary, 13);
					count++;
					g.setColor(color);
					g.fillRect(dragLeft + j*iSquareSize, 135 + i*iSquareSize, 
								  iSquareSize, iSquareSize);
					
					boolean equal = false;
					for (int k = 0; k < 4; k++) {
						int loop = 0;
						for (int l = 0; l < 13; l++) {
							if (binary[l] == EccController.CONTROLLER.ecmo.getMatrixAt(k, l)) {
								loop++;
							}
						}
						if (loop == 13) {
							equal = true;
						}
					}
					if (equal) {
						 g.setColor(Color.black);
						 g.drawRect(dragLeft + j*iSquareSize, 135 + i*iSquareSize,
						 				iSquareSize - 1, iSquareSize - 1);
					}
				}
			}
			g.setColor(Color.black);
			g.drawRect((iPanelWidth - (iSquareSize*iNrOfSqInX))/2 - 1,
				  135 - 1, iSquareSize*iNrOfSqInX + 1, iSquareSize*iNrOfSqInY + 1);
		}
	}	/* paint */

	/**
	 * Liefert die Nummer der angeklickten Farbe
	 * @return Nummer der Farbe
	 */
	private int getColorIndex(int x, int y) {
		iColorIndex = ((x - dragLeft)/iSquareSize) + 
						  ((y - dragUp)/iSquareSize)*iNrOfSqInX;
		return iColorIndex;
	}	/* getColorIndex */

	/**
	 * Setzt die Farbe, die zuletzt im Hyperraum gewaehlt wurde
	 * @param x x-Koordinate des Mouseklicks
	 * @param y y-Koordinate des Mouseklicks
	 * @return Number der Farbe die geaendert wurde
	 */
	private int updateColor(int x, int y) {
		int pos = 4;
		if ((x > p1DropLeft) && (x < p1DropRight)) {
			pos = 0;
		} else if ((x > p2DropLeft) && (x < p2DropRight)) {
			pos = 1;
		} else if ((x > p3DropLeft) && (x < p3DropRight)) {
			pos = 2;
		} else if ((x > p4DropLeft) && (x < p4DropRight)) {
			pos = 3;
		}
		EccController.CONTROLLER.ecmo.setHyperOriginalColorAt(pos, iColorIndex);
		return pos;
	}	/* updateColor */
		
/** *****************************************************************
 * LightweightDragDrop Interface
 ****************************************************************** */
	
	/**
	 * Zeigt, ob die Komponente ein Drag an diesem Ort erlaubt
	 */
	public Object acceptsDrag (MouseEvent event) {
		Point2D.Double mouse = new Point2D.Double(event.getX(), event.getY());
		if ((mouse.x > dragLeft) && (mouse.x < dragRight) && 
			 (mouse.y > dragUp) && (mouse.y < dragDown)) {
			getColorIndex((int)mouse.x, (int)mouse.y);
			iDim = EccController.CONTROLLER.ecmo.getNumOfDim();
			int[] bin = new int[iDim];
			EccController.CONTROLLER.ecmo.getBinary(iColorIndex, bin, iDim);
			String str = "";
			for (int i = 0; i < iDim; i++) {
				str += String.valueOf(bin[i]);
			}
			str = EccConstants.COLOR_CODE + str;
			EccView.setMessageText(str);
			return "Object";
		} else {
			return null;
		}
	}	/* acceptsDrag */

	/**
	 * Zeigt ob die Komponente ein Drop an diesem Ort erlaubt
	 */
	public boolean acceptsDrop (MouseEvent event, Object obj) {
		Point2D.Double mouse = new Point2D.Double(event.getX(), event.getY());
		if ((mouse.x > p1DropLeft) && (mouse.x < p1DropRight) && 
			 (mouse.y > p1DropUp) && (mouse.y < p1DropDown) ||
			 (mouse.x > p2DropLeft) && (mouse.x < p2DropRight) && 
			 (mouse.y > p2DropUp) && (mouse.y < p2DropDown)  ||
			 (mouse.x > p3DropLeft) && (mouse.x < p3DropRight) && 
			 (mouse.y > p3DropUp) && (mouse.y < p3DropDown)  ||
			 (mouse.x > p4DropLeft) && (mouse.x < p4DropRight) && 
			 (mouse.y > p4DropUp) && (mouse.y < p4DropDown)
		) {
			return true;
		} else {
			return false;
		}
	}	/* acceptsDrop */

	/**
	 * Legt das Objekt an diese Stelle ab, falls vorher akzeptiert wurde
	 */
	public void dropTransferObject (MouseEvent event, Object obj) {
		updateColor(event.getX(), event.getY());
		EccController.CONTROLLER.ecmo.setGeneratorFlag(false);
		EccController.CONTROLLER.ecmo.setModifiedFlag(false);
		repaint();
	}	/* dropTransferObject */

	/**
	 * Liefert ein Image des 'Drag'-Objekts
	 */
	public BufferedImage getDragImage (Object dndObject){
		BufferedImage img = null;
		Graphics2D g = (Graphics2D)this.getGraphics();
		Graphics2D g2 = null;
		img = new BufferedImage(iSquareSize,iSquareSize,
										BufferedImage.TYPE_4BYTE_ABGR);
		g2 = (Graphics2D)img.getGraphics();
		Color color = new Color(0,0,0);
		color = EccController.CONTROLLER.ecmo.getHyperErrorColor(iColorIndex);
		g2.setColor(color);
		g2.fillRect(0,0, iSquareSize, iSquareSize);
		g2.dispose();
		g.dispose();
		return img;
	}	/* getDragImage */

}
